; to make a boot sector that will find and load the active
;  partition, parse the FAT12 or FAT16 partition, then load
;  the 'LOADER  SYS' file and jump to it.
;
;
; Sniper
; (aka: Ben Lunt)
; www.fysnet.net
; fys [at] fysnet [dot] net
;
; Two small parts of this code is from Wolfgang's code, since
;  I peeked one time.  Mel Brooks once said, "It's good to be the king" :-)
;
;
; Thank you Wolfgang for participating.  I spent a few nights on this
;  code and enjoyed doing so.
;
;
; This code is assembled with NBASM
;  www.fysnet.net/newbasic.htm
;

.model tiny

outfile 'entry.bin'

.code

include 'entry.inc'

.386

          ; can not assume ds or ss
          ; cs:ip -> 0x07C00
          ; we will set ds to 0x0000
          orgnf 7C00h
          
          ; ds = 0000
          ; first push at 0000:FFFE
          xor  sp,sp
          mov  ds,sp
          mov  ss,sp
          
          ; we can't assume direction
          cld
          
          ; check for extended services
          mov  bx,55AAh
          mov  ah,41h
          int  13h
          jc   short @f
          cmp  bx,0AA55h
          je   short services_okay
          
@@:       mov  si,offset ErrorStr
print_it: lodsb
          or   al,al          ;
@@:       jz   short @b       ; infinite loop when done
          mov  ah,0Eh
          xor  bx,bx
          int  10h
          jmp  short print_it

LoaderStr db 'LOADER  SYS'

ErrorStr  db 'No Disk Services Available.',0
          
          ; extended services found, continue on.
services_okay:
          ; find active partition
          ; this assumes that one of the entries will be active
          mov  bx,(7DBEh - sizeof(S_PART_ENTRY))
@@:       add  bl,sizeof(S_PART_ENTRY)
          dec byte [bx + S_PART_ENTRY->boot_id]   ; if 80h, will now be 7Fh, no sign bit.
          jns short @b                            ; if 00h, will now be FFh, sign bit
          
          ; found active part
@@:       movzx edx,dl      ; clear out high order of edx (drive)
          mov  eax,[bx + S_PART_ENTRY->start]  ; get LBA of starting sector
          push eax    ; save for jump to loader file
          mov  cx,1
          push BPB_SEG
          pop  es
          push es
          pop  gs
          xor  bx,bx
          call read_sectors
          
          ;;;;;;;;;;;;;;;;;;;;;;;;;
          ; gs -> BPB
          
          ; skip reserved sectors
          add  ax,gs:[S_FAT1216_BPB->sect_reserved]
          
          ; read in a fat
          mov  cx,gs:[S_FAT1216_BPB->sect_per_fat]
          push FAT_SEG
          pop  es
          push es      ; fs -> fat
          pop  fs
          ;xor  bx,bx  ; bx still is zero from above
          call read_sectors
          
          ;;;;;;;;;;;;;;;;;;;;;;;;;
          ; fs -> FAT
          
@@:       add  ax,cx
          dec  byte gs:[S_FAT1216_BPB->fats]
          jnz  short @b
          ; we now have a fat at FAT_SEG
          
          ; calculate how many root entries (sectors to load)
          ; may assume that root entries occupy complete sectors
          mov  cx,gs:[S_FAT1216_BPB->root_entrys]
          shr  cx,4   ;  * 32, / 512, = / 16
          push ROOT_SEG
          pop  es     ; es -> ROOT
          ;xor  bx,bx  ; bx still is zero from above
          call read_sectors
          
          ;;;;;;;;;;;;;;;;;;;;;;;;;
          ; es -> ROOT
          
          add  ax,cx  ; lba of first data sector
          
          ; our root is now at ROOT_SEG
          ; find the entry
          ; we may assume the entry will be valid and in the root
          mov  di,(-sizeof(S_FAT_ROOT))
@@:       add  di,sizeof(S_FAT_ROOT)
          push di
          mov  si,offset LoaderStr
          mov  cx,11
          repe
            cmpsb
          pop  di
          jne  short @b
          
          ; found the LOADER file
          ; es:di -> root entry
          ; ax = of first data sector
          ;
          ; ch = 0 from 'mov cx,11' above
          mov  cl,gs:[S_FAT1216_BPB->sect_per_clust]
          
          mov  si,ax  ; save 'base' lba in si
          mov  ax,es:[di + S_FAT_ROOT->strtclst]
          
          ; es now -> place to load file
          push FILE_SEG
          pop  es
          ;xor  bx,bx  ; bx still is zero from above
                    
read_cluster:
          push ax     ; save cluster number
          
          dec  ax     ; 2 based
          dec  ax
          mul  cl     ; sectors per cluster (cl means dx is not modified. cx means dx is zero'd)
          add  ax,si  ; add the base (si)
          call read_sectors
          
          ; increment bx
          mov  ax,512
          mul  cl
          add  bx,ax
          
          pop  di     ; restore cluster number
          mov  bp,di
          
          ; calculate and get next cluster number
          mov  ax,fs:[bp+di] ; 16-bit fat entry?
          cmp  byte gs:[S_FAT1216_BPB->sys_type + 4],'6'  ; 36h
          je   short test_clstr
          
          ; 12-bit fat entry
          shr  di,1
          mov  ax,fs:[bp+di]
          jc   short @f
          shr  ax,4
@@:       and  ah,0Fh
          and  byte [test_clstr + 2],0Fh  ; end flag = 0FF8
          
test_clstr:
          cmp  ax,0FFFFh  ; this gets changed to 00FFFh in the line above if FAT12
          jb   short read_cluster
          
          ; we now have the kernel file at 0x30000
          ; set the register to the following and jump to it
          ;  eax: LBA of partition's base
          ;  ebx: 01020304h
          ;  ecx: DEADBEEFh
          ;  edx: 000000xxh  ; the DL value given at time of boot
          ;  esi: 00000000h
          ;  edi: 00000000h
          ;   cs: 00003000h
          ;   ip: xxxx0000h
          ;   ds: xxxx3000h
          ;   ss: xxxx3000h
          ;   sp: xxxxFFFEh
          pop  eax  ; save LBA of partitions base
          push es   ; es = 3000h
          push es
          pop  ds
          pop  ss
          mov  ebx,01020304h
          mov  ecx,0DEADBEEFh
          xor  esi,esi
          xor  edi,edi
          
          ; at this point, the stack should be at 0000:0000h
          ;  all we have to do is use a single byte push (anything)
          ;  to make it FFFEh
          ;mov  sp,0FFFEh
          push ax
          
          push ds
          push si
          retf


; on entry
;  ax = LBA
;  cx = count
;  dl = drive
; edi = zero
;  es = segment to write to
; saves all registers
read_sectors proc near
          pusha                 ; 1 byte(s)
          push dword 00000000   ; 3
          push 0000             ; 2
          push ax               ; 1
          push es               ; 1
          push bx               ; 1
          push cx               ; 1
          push 0010h            ; 2
          mov  si,sp            ; 2
          mov  ah,42h           ; 2
          int  13h              ; 2

; [4 Jan 2015] A note about the below comment.  I never thought to look, but the
; processor actually ignores the (e)sp pop.  Never needed it, nor looked into it.
; Thanks for showing me, wolfgang.  I learn something new each day. :-)
; so to save 2 bytes, I could have done:
;         popa                  ; 1

          add  sp,16            ; 3 (wish we could do a popa (1 byte) but it will modify sp)
          popa                  ; 1 (   which will mess up the next popa...)
          ret                   ; 1
read_sectors endp

         
.end
